#include <SDL/SDL.h>
#include "lvedes.h"
#include "region.h"
#define _CONTROL_
#include "control.h"


static int  control_set_action(SDL_KeyboardEvent *key, T_VIDEO *video);
static int  control_mouse_down(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video);
static int  control_mouse_down2(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video);
static void control_mouse_up(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video);

static int develop = 1;
void control_main(T_VIDEO *video)
{
  region_t *rg;
  int quit = 0;
  
  SDL_Event event;
 
  //-- select type of event --
  do
  {
    SDL_WaitEvent(&event);   
    switch (event.type)
    {
     case SDL_KEYDOWN:
       quit = control_set_action(&event.key, video);
       break;
              
     case SDL_MOUSEBUTTONDOWN:
       if (!develop)
         quit = control_mouse_down(&event.button, video);
       else
         quit = control_mouse_down2(&event.button, video);
       break;
              
     case SDL_KEYUP:
       rg_region_up(-1, -1);  
       break;

     case SDL_MOUSEBUTTONUP:
       if (!develop)
         control_mouse_up(&event.button, video); 
       break;    

     default: 
       ;
    } 
  } while (!quit);
}

#define N_PT_MAX 2
static int n_pt = 0;
static int x_sav[N_PT_MAX];
static int y_sav[N_PT_MAX];

//== function to process mouse inputs ==
//======================================
void control_save_button(int *x, int *y, int x2, int y2)
{
  SDL_Rect rg_rect, rg_rect1;
  SDL_Surface *sf;
  FILE *fd = NULL;
  
  rg_rect.x = x[0];
  rg_rect.y = y[0];
  
  if (N_PT_MAX == 2)
  {
    rg_rect.w  = x[1] - x[0];
    rg_rect.h  = y[1] - y[0];
  
    rg_rect1.x = x2;
    rg_rect1.y = y2;
    rg_rect1.w = 0;
    rg_rect1.h = 0;
  }
  else
  { 
    rg_rect.w = x2 - x[0];
    rg_rect.h = y2 - y[0];
  }
  
    
   
  sf = SDL_CreateRGBSurface(SDL_SWSURFACE, rg_rect.w, rg_rect.h, 16, 0, 0, 0, 0);
  
  if (sf)
  {
    SDL_BlitSurface(SDL_GetVideoSurface(), &rg_rect, sf, NULL);
    SDL_SaveBMP(sf, PATH_LIB "button.bmp");
    SDL_FreeSurface(sf);
    
    if ((fd = fopen(PATH_LIB "button.rect", "wb")) != NULL)
    {
      printf("Saving: Element at pos (%d, %d) size (%d x %d), Table end at pos (%d, %d)\n", x[0], y[0], rg_rect.w, rg_rect.h, x2, y2);
      fwrite(&rg_rect, sizeof(SDL_Rect), 1, fd);
      
      if (N_PT_MAX == 2) fwrite(&rg_rect1, sizeof(SDL_Rect), 1, fd);
      fclose(fd);
    }
  }
}

static int control_mouse_down2(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video)
{
   if (mouse_b->button == SDL_BUTTON_LEFT) 
   {
     x_sav[n_pt] = mouse_b->x;
     y_sav[n_pt] = mouse_b->y;
    
     if (n_pt == 2) n_pt = 0;
      
     if (n_pt == 0)
       printf("Left top Point at pos (%d,%d) fixed\n", mouse_b->x, mouse_b->y);
     else 
       printf("Right bottom Point at pos (%d,%d) fixed\n", mouse_b->x, mouse_b->y);

     n_pt++;
   }
   else if ((mouse_b->button == SDL_BUTTON_RIGHT) && (n_pt == N_PT_MAX))
   {
     control_save_button(x_sav, y_sav, mouse_b->x, mouse_b->y);
     n_pt = 0;
   }
     
   return 0;
}

static int control_mouse_down(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video)
{
   SDL_KeyboardEvent key;
   region_t          *rg;
   
   if (mouse_b->button == SDL_BUTTON_LEFT) 
   {
     if ((rg = rg_region_fit(mouse_b->x, mouse_b->y)) != NULL)
     {
       if (rg->ksym != -1)
       {
         key.keysym.sym = (SDLKey)rg->ksym;
         return (control_set_action(&key, video));
       }
       
       rg_region_down(rg);
     }
   }
   return 0;
}

static void control_mouse_up(SDL_MouseButtonEvent *mouse_b, T_VIDEO *video)
{
   if (mouse_b->button == SDL_BUTTON_LEFT) 
   {
     rg_region_up(mouse_b->x, mouse_b->y);
   }
}   

//== function to process keyboard inputs ==
//=========================================
static int control_set_action(SDL_KeyboardEvent *key, T_VIDEO *video)
{
  int      rg_id = RG_ID_UNDEF;
  region_t *rg;
   
  //-- select action / function --  
  switch(key->keysym.sym)
  {
    //-- quit program --
    case SDLK_q:
      video->action = A_QUIT;
      return 1;

    //-- toggle develop mode --
    case SDLK_m:
      develop = !develop;
      printf("develop mode switched %s \n", (develop) ? "ON" : "OFF");
      return 0;
 
    default:
      ;
  }
  
  if (rg_id != RG_ID_UNDEF) rg = rg_region_get(rg_id);
 
  rg_region_down(rg);
  
  return 0;
} 
        